diff -ur nginx-1.1.13/src/http/modules/ngx_http_xslt_filter_module.c nginx-1.1.13-new/src/http/modules/ngx_http_xslt_filter_module.c
--- nginx-1.1.13/src/http/modules/ngx_http_xslt_filter_module.c	2010-07-12 14:52:01.000000000 +0200
+++ nginx-1.1.13-new/src/http/modules/ngx_http_xslt_filter_module.c	2012-02-08 13:34:43.000000000 +0100
@@ -10,6 +10,7 @@
 
 #include <libxml/parser.h>
 #include <libxml/tree.h>
+#include <libxslt/variables.h>
 #include <libxslt/xslt.h>
 #include <libxslt/xsltInternals.h>
 #include <libxslt/transform.h>
@@ -48,6 +49,8 @@
     ngx_array_t          sheets;       /* ngx_http_xslt_sheet_t */
     ngx_hash_t           types;
     ngx_array_t         *types_keys;
+    ngx_array_t          string_params; /* ngx_http_complex_value_t */
+    ngx_array_t          params;       /* ngx_http_complex_value_t */
 } ngx_http_xslt_filter_loc_conf_t;
 
 
@@ -75,7 +78,10 @@
 static ngx_buf_t *ngx_http_xslt_apply_stylesheet(ngx_http_request_t *r,
     ngx_http_xslt_filter_ctx_t *ctx);
 static ngx_int_t ngx_http_xslt_params(ngx_http_request_t *r,
-    ngx_http_xslt_filter_ctx_t *ctx, ngx_array_t *params);
+    ngx_http_xslt_filter_ctx_t *ctx,
+    const ngx_http_xslt_filter_loc_conf_t *conf,
+    xsltTransformContextPtr xslt_ctxt,
+    ngx_array_t *params);
 static u_char *ngx_http_xslt_content_type(xsltStylesheetPtr s);
 static u_char *ngx_http_xslt_encoding(xsltStylesheetPtr s);
 static void ngx_http_xslt_cleanup(void *data);
@@ -84,6 +90,13 @@
     void *conf);
 static char *ngx_http_xslt_stylesheet(ngx_conf_t *cf, ngx_command_t *cmd,
     void *conf);
+static char *ngx_http_xslt_add_param(ngx_conf_t *cf, ngx_command_t *cmd,
+    ngx_array_t *params);
+static char *ngx_http_xslt_stylesheet_param(ngx_conf_t *cf, ngx_command_t *cmd,
+    void *conf);
+static char *ngx_http_xslt_stylesheet_string_param(ngx_conf_t *cf,
+    ngx_command_t *cmd,
+    void *conf);
 static void ngx_http_xslt_cleanup_dtd(void *data);
 static void ngx_http_xslt_cleanup_stylesheet(void *data);
 static void *ngx_http_xslt_filter_create_main_conf(ngx_conf_t *cf);
@@ -116,6 +129,20 @@
       0,
       NULL },
 
+    { ngx_string("xslt_param"),
+      NGX_HTTP_MAIN_CONF|NGX_HTTP_SRV_CONF|NGX_HTTP_LOC_CONF|NGX_CONF_TAKE2,
+      ngx_http_xslt_stylesheet_param,
+      NGX_HTTP_LOC_CONF_OFFSET,
+      0,
+      NULL },
+
+    { ngx_string("xslt_string_param"),
+      NGX_HTTP_MAIN_CONF|NGX_HTTP_SRV_CONF|NGX_HTTP_LOC_CONF|NGX_CONF_TAKE2,
+      ngx_http_xslt_stylesheet_string_param,
+      NGX_HTTP_LOC_CONF_OFFSET,
+      0,
+      NULL },
+
     { ngx_string("xslt_types"),
       NGX_HTTP_MAIN_CONF|NGX_HTTP_SRV_CONF|NGX_HTTP_LOC_CONF|NGX_CONF_1MORE,
       ngx_http_types_slot,
@@ -450,6 +477,7 @@
     ngx_uint_t                        i;
     xmlChar                          *buf;
     xmlDocPtr                         doc, res;
+    xsltTransformContextPtr           xctxt;
     ngx_http_xslt_sheet_t            *sheet;
     ngx_http_xslt_filter_loc_conf_t  *conf;
 
@@ -468,14 +496,28 @@
 
     for (i = 0; i < conf->sheets.nelts; i++) {
 
-        if (ngx_http_xslt_params(r, ctx, &sheet[i].params) != NGX_OK) {
+        xctxt = xsltNewTransformContext(sheet[i].stylesheet, doc);
+        if (xctxt == NULL) {
             xmlFreeDoc(doc);
             return NULL;
         }
 
-        res = xsltApplyStylesheet(sheet[i].stylesheet, doc, ctx->params.elts);
+        xctxt->initialContextDoc = doc;
+        xctxt->initialContextNode = (xmlNodePtr) doc;
+
+        if (ngx_http_xslt_params(r, ctx, conf, xctxt,
+                                 &sheet[i].params) != NGX_OK) {
+            xmlFreeDoc(doc);
+            xsltFreeTransformContext(xctxt);
+            return NULL;
+        }
+
+        res = xsltApplyStylesheetUser(sheet[i].stylesheet, doc,
+                                      ctx->params.elts,
+                                      NULL, NULL, xctxt);
 
         xmlFreeDoc(doc);
+        xsltFreeTransformContext(xctxt);
 
         if (res == NULL) {
             ngx_log_error(NGX_LOG_ERR, r->connection->log, 0,
@@ -564,14 +606,92 @@
 
 static ngx_int_t
 ngx_http_xslt_params(ngx_http_request_t *r, ngx_http_xslt_filter_ctx_t *ctx,
+    const ngx_http_xslt_filter_loc_conf_t *conf,
+    xsltTransformContextPtr xctxt,
     ngx_array_t *params)
 {
     u_char                    *p, *last, *value, *dst, *src, **s;
     size_t                     len;
     ngx_uint_t                 i;
-    ngx_str_t                  string;
+    ngx_str_t                  string, name;
     ngx_http_complex_value_t  *param;
 
+    /* location params */
+    param = conf->params.elts;
+
+    for (i = 0; i < conf->params.nelts; i++) {
+
+        if (ngx_http_complex_value(r, &param[i], &string) != NGX_OK) {
+            return NGX_ERROR;
+        }
+
+        value = string.data;
+        len = string.len;
+
+        if ((i & 1) == 0) {
+            ngx_log_debug1(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
+                           "xslt filter param name: \"%s\"", string.data);
+
+            name = string;
+        } else {
+            ngx_log_debug1(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
+                           "xslt filter param value: \"%s\"", string.data);
+
+            dst = value;
+            src = value;
+
+            ngx_unescape_uri(&dst, &src, len, 0);
+
+            *dst = '\0';
+
+            ngx_log_debug1(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
+                           "xslt filter param unescaped: \"%s\"", value);
+
+            if (xsltEvalOneUserParam(xctxt, name.data, value) != 0) {
+
+                ngx_log_error(NGX_LOG_ERR, r->connection->log, 0,
+                              "xslt filter eval failed \"%s\" = \"%s\"",
+                              name.data, string.data);
+
+                /* not failing here, just skipping */
+            }
+        }
+    }
+
+    /* location string params */
+    param = conf->string_params.elts;
+
+    for (i = 0; i < conf->string_params.nelts; i++) {
+
+        if (ngx_http_complex_value(r, &param[i], &string) != NGX_OK) {
+            return NGX_ERROR;
+        }
+
+        if ((i & 1) == 0) {
+            ngx_log_debug1(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
+                           "xslt filter param name: \"%s\"", string.data);
+
+            name = string;
+        } else {
+            ngx_log_debug1(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
+                           "xslt filter param value: \"%s\"", string.data);
+
+            if (xsltQuoteOneUserParam(xctxt, name.data, string.data) != 0) {
+
+                ngx_log_error(NGX_LOG_ERR, r->connection->log, 0,
+                              "xslt filter quoting failed \"%s\" = \"%s\"",
+                              name.data, string.data);
+
+                /* do not fail here, just skip */
+                continue;
+            }
+
+            ngx_log_debug1(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
+                           "xslt filter param quoted: \"%s\"", value);
+        }
+    }
+
+    /* per-stylesheet params */
     param = params->elts;
 
     for (i = 0; i < params->nelts; i++) {
@@ -865,6 +985,69 @@
 }
 
 
+static char *
+ngx_http_xslt_add_param(ngx_conf_t *cf, ngx_command_t *cmd,
+    ngx_array_t *params)
+{
+    ngx_uint_t                        i;
+    ngx_http_complex_value_t         *param;
+    ngx_http_compile_complex_value_t  ccv;
+    ngx_str_t                        *value;
+
+    value = cf->args->elts;
+
+    /* alloc array */
+    if (params->elts == NULL) {
+        if (ngx_array_init(params, cf->pool, 2,
+                           sizeof(ngx_http_complex_value_t))
+            != NGX_OK)
+        {
+            return NGX_CONF_ERROR;
+        }
+    }
+
+    for (i = 1; i <= 2; i++) {
+
+         param = ngx_array_push(params);
+         if (param == NULL) {
+             return NGX_CONF_ERROR;
+         }
+
+         ngx_memzero(&ccv, sizeof(ngx_http_compile_complex_value_t));
+
+         ccv.cf = cf;
+         ccv.value = &value[i];
+         ccv.complex_value = param;
+         ccv.zero = 1;
+
+         if (ngx_http_compile_complex_value(&ccv) != NGX_OK) {
+             return NGX_CONF_ERROR;
+         }
+    }
+
+    return NGX_CONF_OK;
+}
+
+
+static char *
+ngx_http_xslt_stylesheet_param(ngx_conf_t *cf, ngx_command_t *cmd, void *conf)
+{
+    ngx_http_xslt_filter_loc_conf_t  *xlcf = conf;
+
+    return ngx_http_xslt_add_param(cf, cmd, &xlcf->params);
+}
+
+
+static char *
+ngx_http_xslt_stylesheet_string_param(ngx_conf_t *cf, ngx_command_t *cmd,
+    void *conf)
+{
+    ngx_http_xslt_filter_loc_conf_t  *xlcf = conf;
+
+    return ngx_http_xslt_add_param(cf, cmd, &xlcf->string_params);
+}
+
+
 static void
 ngx_http_xslt_cleanup_dtd(void *data)
 {
@@ -944,6 +1127,11 @@
         conf->sheets = prev->sheets;
     }
 
+    if ((conf->params.nelts + conf->string_params.nelts) == 0) {
+        conf->params = prev->params;
+        conf->string_params = prev->string_params;
+    }
+
     if (ngx_http_merge_types(cf, &conf->types_keys, &conf->types,
                              &prev->types_keys, &prev->types,
                              ngx_http_xslt_default_types)